/*
 * DLOG
 * Copyright (c) 2005-2008, The Android Open Source Project
 * Copyright (c) 2012-2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _LOG_FILE_H
#define _LOG_FILE_H

#include <logprint.h>
#include <queued_entry.h>

/* Logfile rotation */
#define DEFAULT_ROTATE_SIZE_KB 1024
#define DEFAULT_ROTATE_NUM_FILES 3

#define BtoKiB(x)	((x) >> 10)

struct log_file;

struct log_file {
	char *path;
	int fd;
	int should_close;
	int size;
	size_t rotate_size_kbytes;
	size_t max_rotated;
	struct log_format format;
	bool colors_auto;
	int32_t prev_sec;  /* previously written entry's timestamp, seconds */
	int32_t prev_nsec; /* previously written timestamp, nanoseconds */
};

#ifdef __cplusplus
extern "C" {
#endif

void logfile_init(struct log_file *l_file);
void logfile_free(struct log_file *l_file);
void logfile_set_fd(struct log_file *l_file, int fd, int should_close);
int logfile_set_path(struct log_file *l_file, const char *path);
int logfile_open(struct log_file *l_file);
int logfile_rotate_needed(struct log_file *l_file);
void logfile_do_rotate(struct log_file *file);
int logfile_write_with_rotation(const struct logger_entry *e, struct log_file *file, enum sorting_order sort_by);

typedef int write_callback(const struct logger_entry *e, void *userdata);

struct logfile_callback_data
{
	struct log_file *file;
	enum sorting_order sort_by;
};
int logfile_callback(const struct logger_entry *e, void *userdata);

#ifdef __cplusplus
}
#endif

#endif /* _LOG_FILE_H */
