/*
 * DLOG
 * Copyright (c) 2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _LOGCONFIG_H_
#define _LOGCONFIG_H_

#include <stdbool.h>

#define MAX_CONF_KEY_LEN   32
#define MAX_CONF_VAL_LEN   256
#define MAX_CONF_ENTRY_LEN (MAX_CONF_KEY_LEN + MAX_CONF_VAL_LEN + 2) // +2 for the delimiter and newline

#define DEFAULT_CONFIG_SUFFIX	".conf"

struct log_conf_entry;

struct log_config {
	struct log_conf_entry *begin;
	struct log_conf_entry *last;
};

/**
 * @brief Config iteration callback
 * @param[in] key Key of an entry
 * @param[in] value Value of the entry
 * @param[in,out] userdata User data
 * @return false if processing done, else true if keep processing
 * @see log_config_foreach
 */

typedef void (*configIter) (const char *key, const char *value, void *userdata);

#ifdef __cplusplus
extern "C" {
#endif

void log_config_set(struct log_config *config, const char *key, const char *value);
bool log_config_remove(struct log_config *config, const char *key);
const char *log_config_get(const struct log_config *config, const char *key);
int log_config_get_int(const struct log_config *config, const char *key, int default_val);
bool log_config_get_boolean(const struct log_config *config, const char *key, bool default_val);
int log_config_read(struct log_config *config);
int log_config_read_file(struct log_config *config, const char *filename);
void log_config_free(struct log_config *config);
void log_config_copy(struct log_config *dst, const struct log_config *src);
void log_config_foreach(const struct log_config *config, configIter iter, void *userdata);
enum sorting_order get_order_from_config(const struct log_config *conf);
enum sorting_order get_proper_sort_by(enum sorting_order sort_by, bool is_pipe, const struct log_config *conf, bool *sorting_needed);

#ifdef __cplusplus
}
#endif

#endif /* _LOGCONFIG_H_ */
