#pragma once

/*
 * Copyright (c) 2018, Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <limits.h>

#include <logconfig.h>
#include <log_file.h>
#include <queued_entry.h>

#define IS_VECTOR_SIZE_SORTABLE(x) (x > 1)

#define DUMP_CONTINUOUS 0
#define DUMP_INFINITE SIZE_MAX

#define DEFAULT_SORT_BUFFER_SIZE 131072 // the size (in ENTRIES!) of the sorting buffer

struct sort_vector {
	struct logger_entry **data;
	int begin;
	int end;
	size_t size;
	long timeout;
	size_t dump;
	enum sorting_order sort_by;
	bool old_logs_dumped;
	struct timespec start;	/* mark start to detect the moment when all "old" logs (existing before util has been invoked) have been dumped */
};

void sort_vector_init(struct sort_vector *logs);
int sort_vector_finalize(struct sort_vector *logs);
void sort_vector_free(struct sort_vector *logs);
struct logger_entry *sort_vector_back(struct sort_vector *logs);
int sort_vector_enlarge(struct sort_vector *logs);
int sort_vector_empty(struct sort_vector *logs);
int sort_vector_full(struct sort_vector *logs);
int sort_vector_used_size(struct sort_vector *logs);
void sort_vector_pop(struct sort_vector *logs);
void sort_vector_apply_config(struct sort_vector *logs, const struct log_config *config);
int sort_vector_push(struct sort_vector *logs, struct logger_entry *p, write_callback callback, void *userdata);
long sort_vector_time_span(struct sort_vector *logs);
int sort_vector_flush(struct sort_vector *logs, write_callback callback, void *userdata, bool force);
