%define _with_dlog_dynamic_filters on
%bcond_with dlog_dynamic_filters

Name:       dlog
Summary:    Logging service
Version:    5.0.6
Release:    1
Group:      System/Libraries
License:    Apache-2.0, MIT
Source0:    %{name}-%{version}.tar.gz
Source101:  packaging/dlog.manifest
Source501:  packaging/01-dlog.rules
Source502:  packaging/logger-devices.conf

# NOTE: hardcoded instead of a build dependency on libudev.
#       Needed to prevent a cyclic build dependency in case
#       dlog is used in a package dependent on libsystemd,
#       since libudev is a part of it
%define _udevrulesdir %{_prefix}/lib/udev/rules.d

BuildRequires: autoconf
BuildRequires: automake
BuildRequires: libtool
BuildRequires: pkgconfig(capi-base-common)
BuildRequires: pkgconfig(libtzplatform-config)
BuildRequires: lcov

%description
dlog API library

%package -n libdlog
Summary:    Logging service dlog API
Group:      Development/Libraries
Requires(pre): coreutils
Requires(post): coreutils
Requires(post): /sbin/ldconfig
Requires(post): /usr/bin/chsmack
Requires(post): /usr/bin/systemctl
Requires(posttrans): coreutils
Requires(posttrans): /usr/bin/chsmack
Requires(postun): /sbin/ldconfig
Requires: %{name}-config-persistent-logs

%description -n libdlog
dlog API library. If dlogbackend isn't installed, pipe backend is selected.

%package -n libdlog-devel
Summary:    Logging service dlog API
Group:      Development/Libraries
Requires:   lib%{name} = %{?epoch:%{epoch}:}%{version}-%{release}

%description -n libdlog-devel
dlog API library

%package config-backend-logger
Summary:	Android-logger backend selection for dlog
Requires:	lib%{name} = %{?epoch:%{epoch}:}%{version}-%{release}

%description config-backend-logger
Change dlog backend to logger backend. Require kernel support (Android-logger)

%package -n dlogbackend-logger
Summary:	transitional dummy package
Requires:	%{name}-config-backend-logger = %{?epoch:%{epoch}:}%{version}-%{release}

%description -n dlogbackend-logger
Dummy packate to ease transition from dlogbackend-logger to dlog-config-backend-logger.

%package -n dlogutil
Summary:    print log data to the screen
Group:      Development/Libraries
Requires:   lib%{name} = %{?epoch:%{epoch}:}%{version}-%{release}

%description -n dlogutil
Utilities for print log data

%package config-syslog
License:    Apache-2.0
Summary:    Logger: syslog-source logging via dlog
Group:      Base/Startup
Requires:   lib%{name} = %{?epoch:%{epoch}:}%{version}-%{release}
Provides:   tizen-log-syslog

%description config-syslog
Systemd unit for syslog socket configation to use with dlog

%package config-persistent-logs
Summary: enable persistent storage of logs

%description config-persistent-logs
Enable persistent logs in dlog (stored in /var/log/dlog/)

%package doc
Summary: dlog documentation
BuildRequires: doxygen

%description doc
Documentation for dlog

%package tests
Summary: dlog integration tests
Requires: lib%{name} = %{?epoch:%{epoch}:}%{version}-%{release}
Requires(post): /usr/bin/chsmack

%description tests
Integration tests for dlog.

%prep
%setup -q

%build
if echo "$CFLAGS" | grep -q -- '-Wl,--as-needed'; then
  LDFLAGS="${LDFLAGS} -Wl,--as-needed"
fi
CFLAGS="$(echo $CFLAGS | sed -e 's/-Wl,--as-needed//g') -Wno-gnu-variable-sized-type-not-at-end"
CXXFLAGS="$(echo $CXXFLAGS | sed -e 's/-Wl,--as-needed//g')"

cp %{SOURCE101} .

%autogen --disable-static
%configure --disable-static \
%if %{without dlog_dynamic_filters}
--disable-dynamic-filters
%else
--enable-dynamic-filters
%endif

make %{?jobs:-j%jobs} CFLAGS+='-D__DLOG_VERSION=\"%{version}\" -D__DLOG_RELEASE=\"%{release}\"'

# autoconf adds a bunch of flags, including -O2.
# Clear those so that test coverage can be measured properly
make check CFLAGS='-Wno-gnu-variable-sized-type-not-at-end -Wno-logical-op-parentheses -D__DLOG_VERSION=\"%{version}\" -D__DLOG_RELEASE=\"%{release}\"'

make dox

%install
rm -rf %{buildroot}

%make_install
mkdir -p %{buildroot}/usr/bin/

mkdir -p %{buildroot}/usr/share/doc/dlog
cp -r documentation/* %{buildroot}/usr/share/doc/dlog

mkdir -p %{buildroot}%{_unitdir}/basic.target.wants/
mkdir -p %{buildroot}%{_unitdir}/sockets.target.wants/
ln -s ../dlog_logger.service %{buildroot}%{_unitdir}/basic.target.wants/dlog_logger.service
ln -s ../dlog_dev-log.socket %{buildroot}%{_unitdir}/sockets.target.wants/dlog_dev-log.socket

mkdir -p %{buildroot}%{_udevrulesdir}
install -m 0644 %SOURCE501 %{buildroot}%{_udevrulesdir}/01-dlog.rules

mkdir -p %{buildroot}%{_unitdir}/dlog_logger.service.d
install -m 0644 %SOURCE502 %{buildroot}%{_unitdir}/dlog_logger.service.d/logger-devices.conf

mkdir -p %{buildroot}/var/log/dlog

mkdir -p %{buildroot}/var/lib/dlog-tests
mkdir -p %{buildroot}/usr/share/dlog-filters.conf.test

%if %{without dlog_dynamic_filters}
rm -f %{buildroot}%{_bindir}/dlogctl
sed -i -e 's,^\(dynamic_config_path=\),#\1,g' -e 's,^\(limiter_apply_to_all_buffers=\),#\1,g' %{buildroot}%{_sysconfdir}/dlog.conf
%endif

%files -n dlogutil
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%attr(750,log,log) %{_bindir}/dlogutil
%attr(755,log,log) %{_bindir}/dlogsend
%if %{with dlog_dynamic_filters}
%attr(750,log,log) %{_bindir}/dlogctl
%endif

%post -n dlogutil
%if %{with dlog_dynamic_filters}
chsmack %{_bindir}/dlogctl -a "System::Tools"
%endif

%files -n libdlog
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%{_libdir}/libdlog.so.0
%{_libdir}/libdlog.so.0.0.0
%{_sysconfdir}/dlog.conf
%{_sysconfdir}/dlog.conf.d/20-pipe.conf
%attr(755,log,log) /var/log/dlog
%attr(750,log,log) %{_bindir}/dlog_logger
%{_unitdir}/basic.target.wants/dlog_logger.service
%{_unitdir}/dlog_logger.service
/usr/lib/tmpfiles.d/dlog-run.conf
%{_udevrulesdir}/01-dlog.rules

%post -n libdlog
/sbin/ldconfig
systemctl daemon-reload
chsmack -a System /var/log/dlog

%postun -n libdlog -p /sbin/ldconfig

%files -n libdlog-devel
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%{_includedir}/dlog/dlog.h
%{_includedir}/dlog/dlog-internal.h
%{_libdir}/pkgconfig/dlog.pc
%{_libdir}/libdlog.so

%files config-backend-logger
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%{_sysconfdir}/dlog.conf.d/25-logger.conf
%{_unitdir}/dlog_logger.service.d/logger-devices.conf

%files -n dlogbackend-logger

%files config-syslog
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%{_unitdir}/sockets.target.wants/dlog_dev-log.socket
%{_unitdir}/dlog_dev-log.socket

%files config-persistent-logs
%{_sysconfdir}/dlog.conf.d/10-persistent-logs.conf

%files doc
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
/usr/share/doc/dlog

%files tests
%manifest dlog.manifest
%license LICENSE.APACHE2.0 LICENSE.MIT
%{_bindir}/dlog_test
/usr/share/doc/dlog/README.testsuite
%{_libexecdir}/libdlog/perf_libdlog
%{_libexecdir}/libdlog/test_libdlog
%{_libexecdir}/libdlog/test_filters
%{_datadir}/dlog-pipe.conf.test
%{_datadir}/dlog-logger.conf.test
%{_sysconfdir}/dlog.conf.d/99-dlog-logger.disable-platform-logging-for-testsuite.conf
%dir %attr(755,log,log) /var/lib/dlog-tests
%dir %attr(755,log,log) /usr/share/dlog-filters.conf.test

%post tests
chsmack -a System /var/lib/dlog-tests
echo "Warning! Installation of dlog-tests will break platform logging."
echo "This is done intentionally, to make logger backend testing possible."
echo "To restore normal behavior, please uninstall dlog-tests package."
echo ""
echo "For more information please read the /usr/share/doc/dlog/README.testsuite file"
echo ""
echo "A reboot is required to properly configure logger backend for testing."
