// C
#include <assert.h>

// DLog
#include <fd_info.h>

static bool use_real_free;
static void *expect_free1, *expect_free2;
void __real_free(void *ptr);
void __wrap_free(void *ptr)
{
	if (use_real_free)
		__real_free(ptr);
	else
		assert(ptr == expect_free1 || ptr == expect_free2);
}

static bool use_real_malloc;
static void *malloc_ret;
void *__real_malloc(size_t size);
void *__wrap_malloc(size_t size)
{
	return use_real_malloc ? __real_malloc(size) : malloc_ret;
}

static bool use_real_close;
int __real_close(int fd);
int __wrap_close(int fd)
{
	if (use_real_close)
		return __real_close(fd);

	assert(fd == 0xFD);
	return 0;
}

static bool sv_flushed;
int callback(const struct logger_entry *entry, void *userdata)
{
	assert(entry == (struct logger_entry *) 0xBADFEEL);
	assert(userdata == (struct log_file *) 0xDEFACED);

	sv_flushed = true;
	return 0;
}

static bool sv_pushed;
static int sv_push_retval;
int __wrap_sort_vector_push(struct sort_vector *logs, struct logger_entry *p, write_callback comp_callback, void *userdata)
{
	assert(p == (struct logger_entry *) 0xBADFEEL);
	assert(comp_callback == callback);
	assert(userdata == (struct log_file *) 0xDEFACED);

	sv_pushed = true;
	return sv_push_retval;
}

struct fd_info G_fdi;

static bool fail_extract;
struct logger_entry *test_extract(struct fd_info *fdi)
{
	assert(fdi == &G_fdi);
	return fail_extract ? NULL : (struct logger_entry *) 0xBADFEEL;
}

void __wrap_extract_timestamp(const struct logger_entry *le, enum sorting_order stamp_type, struct timespec *ts)
{
	assert(le == (struct logger_entry *)0xBADFEEL);
	ts->tv_sec = 1;
	ts->tv_nsec = 1;
}

static struct fd_info *expected_destroyee;
void test_destroy(struct fd_info *fdi)
{
	assert(fdi == expected_destroyee);
}

static bool should_print_line = true;
bool __wrap_log_should_print_line(log_filter *p_filter, const struct logger_entry *entry)
{
	assert(p_filter == (log_filter *)404);
	assert(entry == (struct logger_entry *)0xBADFEEL);
	return should_print_line;
}

int main()
{
	struct fd_ops fops = {
		.destroy = test_destroy,
		.extract_entry = test_extract,
	};

	malloc_ret = NULL;
	assert(NULL == fdi_create(&fops, "foobar"));
	fdi_free(NULL); // shouldn't explode

	malloc_ret = &G_fdi;
	G_fdi = (struct fd_info) {
		.fd = 52,
		.priv_data = (void *) 0x16180339,
		.name = (const char *) 0x27182818,
		.ops = (struct fd_ops *) 0x31415926,
	};
	assert(&G_fdi == fdi_create(&fops, "foobar"));
	assert(-1 == G_fdi.fd);
	assert(!G_fdi.priv_data);
	assert(!strcmp(G_fdi.name, "foobar"));
	assert(G_fdi.ops == &fops);
	G_fdi.fd = 0xFD;

	use_real_malloc = true;
	struct fd_info *const local_fdi = malloc(sizeof *local_fdi);
	assert(local_fdi);
	memcpy(local_fdi, &G_fdi, sizeof *local_fdi);
	expect_free1 = local_fdi;
	expected_destroyee = local_fdi;
	fdi_free(local_fdi);
	__real_free(local_fdi);
	use_real_malloc = false;

	struct fd_info **fdi_arr = NULL;
	fdi_array_free(&fdi_arr);

	fdi_arr = calloc(3, sizeof *fdi_arr);
	assert(fdi_arr);
	fdi_arr[0] = &G_fdi;
	expected_destroyee = &G_fdi;
	expect_free1 = fdi_arr;
	expect_free2 = &G_fdi;
	fdi_array_free(&fdi_arr);

	/* This memory is not faked and actually lives on the heap (note that `malloc`
	 * is wrapped but `calloc` is not) so it needs to be handled manually. */
	__real_free(fdi_arr);

	fdi_arr = calloc(3, sizeof *fdi_arr);
	assert(fdi_arr);
	fdi_arr[1] = &G_fdi;
	expect_free1 = fdi_arr;
	expect_free2 = NULL; // shouldn't reach [1] &G_fdi because [0] NULL stops iteration
	fdi_array_free(&fdi_arr);
	__real_free(fdi_arr);

	struct sort_vector sv;

	should_print_line = false;
	expect_free1 = (void *) 0xBADFEEL;
	assert(0 == fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	should_print_line = true;

	fail_extract = true;
	assert(-ENOMEM == fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	fail_extract = false;

	sv.size = 99;
	expect_free1 = NULL;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(sv_pushed);
	assert(!sv_flushed);
	sv_pushed = false;

	sv_push_retval = -83;
	expect_free1 = &G_fdi;
	assert(-83 == fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(sv_pushed);
	assert(!sv_flushed);
	sv_push_retval = 0;

	sv.old_logs_dumped = false;
	sv.start.tv_sec = 2;
	sv.start.tv_nsec = 0;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(!sv.old_logs_dumped);

	sv.old_logs_dumped = false;
	sv.start.tv_sec = 1;
	sv.start.tv_nsec = 1;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(!sv.old_logs_dumped);

	sv.old_logs_dumped = false;
	sv.start.tv_sec = 1;
	sv.start.tv_nsec = 0;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(sv.old_logs_dumped);

	sv.old_logs_dumped = false;
	sv.start.tv_sec = 0;
	sv.start.tv_nsec = 2;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(sv.old_logs_dumped);

	sv_pushed = false;
	sv.size = 0;
	expect_free1 = (void *) 0xBADFEEL;
	assert(!fdi_push_log(&G_fdi, &sv, callback, (void *) 0xDEFACED, (log_filter *) 404));
	assert(!sv_pushed);
	assert(sv_flushed);

	use_real_malloc = true;
	use_real_free = true;
	use_real_close = true;
}
