// C
#include <assert.h>

// DLog
#include <dlog_ioctl.h>
#include <fdi_logger.h>

static int open_buf_ret;
static const char *open_buf_path;
int __wrap_logger_open_buffer_from_config_get_path(int buf_id, const struct log_config *conf, int open_flags, int *fd, char *actual_path, size_t path_size)
{
	assert(buf_id == LOG_ID_RADIO);
	assert(conf == (struct log_config *) 0xABDUL);
	assert(open_flags == O_RDWR);

	if (open_buf_path)
		strncpy(actual_path, open_buf_path, path_size);
	if (open_buf_ret > 0)
		*fd = 0xFD;
	return open_buf_ret;
}

static size_t filters_added;
static char *const filter_strs[] = {"H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne"};
int __wrap_log_add_filter_string(log_filter *p_filter, const char *filterString)
{
	assert(filterString);
	assert(p_filter == (log_filter *) 0xCA5CADE);

	++filters_added;
	for (size_t i = 0; i < NELEMS(filter_strs); ++i)
		if (!strcmp(filter_strs[i], filterString))
			return 0;

	assert(false);
}

void __wrap_parse_androidlogger_message(struct android_logger_entry *ale, struct logger_entry *le, size_t dgram_size) { }
void __wrap_copy_recv_timestamp(struct logger_entry *le) { }

static bool use_real_close;
int __real_close(int fd);
int __wrap_close(int fd)
{
	if (use_real_close)
		return __real_close(fd);

	assert(fd == 0xFD);
	return 0;
}

static const size_t READS[] = {
	113, 259, 325, 207, 130, 281, 261, 169, 179, 379, 394, 390, 155, 228, 399,
	288, 341, 1, 320, 221, 185, 255, 88, 108, 391, 299, 243, 382, 7, 41, 163, 7,
	62, 288, 279, 350, 139, 29, 167, 28, 230, 172, 383, 329, 369, 263, 391, 37,
	383, 186, 48, 369, 238, 227, 99, 217, 261, 313, 294, 253, 151, 325, 326, 21,
	210, 350, 388, 97, 347, 348, 340, 3, 185, 98, 212, 74, 144, 52, 110, 202, 2,
	202, 282, 103, 90, 150, 182, 145, 104, 159, 49, 69, 69, 69, 163, 184, 358,
	50, 124, 205, 187, 139, 329, 361, 224, 263, 345, 99, 87, 127, 184, 231, 290,
	192, 31, 239, 370, 324, 362, 58, 54, 214, 115, 352, 228, 161, 137, 240, 207,
	273, 200, 154, 364, 310, 38, 332, 92, 334, 238, 52, 181, 205, 343, 312, 230,
	164, 306, 187, 181, 346, 233, 274, 325, 129, 79, 282, 82, 304, 48, 95, 312,
	151, 87, 212, 308, 96, 201, 274, 92, 212, 283, 173, 74, 237, 137, 49, 239,
	155, 192, 222, 88, 267, 229, 383, 96, 3, 46, 146, 182, 182, 155, 151, 106,
	195, 226, 112, 231, 294, 84, 115, 386, 116, 243, 54, 396, 324, 240, 203, 89,
};
static size_t read_cnt = 0;
static bool use_real_read;
static bool fail_read;
ssize_t __real_read(int fd, void *buf, size_t count);
ssize_t __wrap_read(int fd, void *buf, size_t count)
{
	if (use_real_read)
		return __real_read(fd, buf, count);

	if (fail_read) {
		errno = 334;
		return -1;
	}

	if (read_cnt == NELEMS(READS))
		return 0;

	const size_t current_read = READS[read_cnt++];
	assert(current_read < count);
	((char *)buf)[current_read] = '\0';
	return current_read;
}

static int fail_malloc;
void *__real_malloc(size_t size);
void *__wrap_malloc(size_t size)
{
	void *const ret = (fail_malloc & 1) ? NULL : __real_malloc(size);
	fail_malloc >>= 1;
	return ret;
}

static bool fail_calloc;
void *__real_calloc(size_t nmemb, size_t size);
void *__wrap_calloc(size_t nmemb, size_t size)
{
	return fail_calloc ? NULL : __real_calloc(nmemb, size);
}

static int expected_ioctl;
static int ioctl_ret;
int __real_ioctl(int fd, int request, char *argp);
int __wrap_ioctl(int fd, int request, char *argp)
{
	assert(fd == 0xFD);
	assert(expected_ioctl == request);

	if (ioctl_ret < 0) {
		errno = -ioctl_ret;
		return -1;
	}

	return ioctl_ret;
}

int main()
{
	struct fd_info fdi = {
		.fd = -1,
		.priv_data = NULL,
	};
	list_head used_paths = NULL;

	ops_logger.destroy(&fdi);

	fail_malloc = 1;
	assert(-ENOMEM == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));

	open_buf_ret = -536;
	assert(-536 == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));
	open_buf_ret = 0;
	assert(-EINVAL == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));

	open_buf_path = "abc";
	open_buf_ret = 1;
	fail_malloc = 2;
	assert(-ENOMEM == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));
	assert(fdi.fd < 0);
	assert(!fdi.priv_data);

	fail_calloc = true;
	assert(-ENOMEM == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));
	assert(fdi.fd < 0);
	assert(!fdi.priv_data);
	fail_calloc = false;

	assert(!ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));
	assert(fdi.fd == 0xFD);
	assert(fdi.priv_data);

	ops_logger.destroy(&fdi);
	assert(!fdi.priv_data);
	fdi.fd = -1;

	assert(-EALREADY == ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));
	assert(fdi.fd == -1);
	assert(!fdi.priv_data);

	open_buf_path = "def";
	assert(!ops_logger.create(&fdi, (struct log_config *) 0xABDUL, "radio", &used_paths));

	expected_ioctl = LOGGER_FLUSH_LOG;
	ioctl_ret = -77;
	assert(-77 == ops_logger.clear(&fdi));
	ioctl_ret = 15;
	assert(0 == ops_logger.clear(&fdi));

	uint32_t sz;
	expected_ioctl = LOGGER_GET_LOG_BUF_SIZE;
	ioctl_ret = -EALREADY;
	assert(-EALREADY == ops_logger.getsize(&fdi, &sz));
	ioctl_ret = 2019;
	assert(0 == ops_logger.getsize(&fdi, &sz));
	assert(2019 == sz);

	list_head filters = NULL;
	expected_ioctl = -1;
	assert(!ops_logger.prepare_print(&fdi, DUMP_CONTINUOUS, filters, (log_filter *) 0xCA5CADE));

	expected_ioctl = LOGGER_GET_LOG_LEN;
	ioctl_ret = -67;
	assert(-67 == ops_logger.prepare_print(&fdi, 123, filters, (log_filter *) 0xCA5CADE));
	expected_ioctl = LOGGER_GET_LOG_LEN;
	ioctl_ret = 0;
	assert(1 == ops_logger.prepare_print(&fdi, 123, filters, (log_filter *) 0xCA5CADE));

	ioctl_ret = 128;
	filters_added = 0;
	for (size_t i = 0; i < NELEMS(filter_strs); ++i)
		list_add(&filters, filter_strs[i]);

	assert(!ops_logger.prepare_print(&fdi, 123, filters, (log_filter *) 0xCA5CADE));
	assert(filters_added == NELEMS(filter_strs));

	fail_read = true;
	assert(-334 == ops_logger.read(&fdi));
	fail_read = false;

	size_t total_reads = 0;
	for (size_t i = 0; i < NELEMS(READS) + 15; ++i) {
		assert(!ops_logger.has_log(&fdi));
		assert(NULL == ops_logger.extract_entry(&fdi));
		assert(NULL == ops_logger.peek_entry(&fdi));
		assert(NULL == ops_logger.extract_entry(&fdi));
		fail_malloc = !(rand() % 4) * 3;
		ops_logger.read(&fdi);
		if (!fail_malloc && i < NELEMS(READS) && READS[i] > sizeof(struct android_logger_entry)) {
			assert(ops_logger.has_log(&fdi));
			assert(-EAGAIN == ops_logger.read(&fdi));
			const struct logger_entry *const le = ops_logger.peek_entry(&fdi);
			assert(le);
			assert(le == ops_logger.peek_entry(&fdi));
			assert(le == ops_logger.extract_entry(&fdi));
			free((struct logger_entry *) le);
		}
		fail_malloc = 0;
		assert(!ops_logger.has_log(&fdi));
		assert(NULL == ops_logger.peek_entry(&fdi));
		assert(NULL == ops_logger.extract_entry(&fdi));
	}

	ops_logger.destroy(&fdi);
}
