#include <logprint.h>
#include <queued_entry.h>
#include <stdio.h>
#include <assert.h>

void assert_result_buffer(char const *msg, int expected_outcome, struct logger_entry_with_msg *lem)
{
	char buffer[LOG_MAX_PAYLOAD_SIZE];
	const int len = snprintf(buffer, sizeof buffer, "%s", msg);
	assert(parse_kmsg_message(buffer, lem, len) == expected_outcome);
}

void assert_result(char const *msg, int expected_outcome)
{
	struct logger_entry_with_msg lem;
	assert_result_buffer(msg, expected_outcome, &lem);
}

void assert_entry(char const * msg, const struct logger_entry_with_msg *expected)
{
	struct logger_entry_with_msg lem;
	assert_result_buffer(msg, 0, &lem);

	assert(expected->header.pid == lem.header.pid);
	assert(expected->header.tid == lem.header.tid);

	const char *const tag_expected = log_buffer_get_tag(&expected->header);
	const char *const tag_actual   = log_buffer_get_tag(&lem.header);
	const char *const msg_expected = log_buffer_get_message(&expected->header);
	const char *const msg_actual   = log_buffer_get_message(&lem.header);
	assert(tag_expected);
	assert(tag_actual);
	assert(msg_expected);
	assert(msg_actual);
	assert(!strcmp(tag_expected, tag_actual));
	assert(!strcmp(msg_expected, msg_actual));
}

int main()
{
	struct logger_entry_with_msg lem;
	assert_result(""             , -EINVAL);
	assert_result("invalid"      , -EINVAL);
	assert_result("1,2"          , -EINVAL);
	assert_result("1,2,3"        , -EINVAL);
	assert_result("1,2,3,-;foo"  , -EINVAL);
	assert_result("1,2,3,4,5,6,7", -EINVAL);
	assert_result(";bla"         , -EINVAL);

	assert_result("1,2,99999999999999999999999999999999999999,4", -ERANGE);

	lem.header.pid = 0;
	lem.header.tid = 0;

	lem.header.priority = DLOG_FATAL;
	lem.header.tag_len = snprintf(lem.msg, sizeof lem.msg - 1, "%s", "SYSLOG_KERN");
	lem.header.len = lem.header.tag_len + 1; // plus NULL delimiter
	lem.header.len += 1 + snprintf(lem.msg + lem.header.len, sizeof lem.msg - 1 - lem.header.len, "%s", "Some message.");

	lem.header.len += sizeof(struct logger_entry);
	lem.header.padding = 0;

	assert_entry("1,234,56789,-;Some message.\n _SOME_IRRELEVANT_TAG=BLA\n", &lem);

	lem.header.tag_len = snprintf(lem.msg, sizeof lem.msg - 1, "%s", "SYSLOG-UNKNOWN");
	lem.header.len = lem.header.tag_len + 1;
	lem.header.len += 1 + snprintf(lem.msg + lem.header.len, sizeof lem.msg - 1 - lem.header.len, "%s", "Some message.");
	lem.header.len += sizeof lem.header;
	assert_entry("241,234,56789,-;Some message.\n _SOME_IRRELEVANT_TAG=BLA\n", &lem);

	return 0;
}
