#include <assert.h>
#ifdef USE_ANDROID_MONOTONIC
#include <time.h>
#endif /* USE_ANDROID_MONOTONIC */

#include <sys/uio.h>

#include <dlog.h>
#include <logcommon.h>
#include <logconfig.h>
#ifdef USE_ANDROID_MONOTONIC
#include <queued_entry.h>
#endif /* USE_ANDROID_MONOTONIC */

int (*write_to_log)(log_id_t, log_priority, const char *tag, const char *msg) = NULL;
void (*destroy_backend)() = NULL;
extern void __dlog_init_android(const struct log_config *conf);

static struct log_config CONF;

static bool fail_open;
static size_t closed = 0;
static int fake_buf_fd[LOG_ID_MAX];
int __wrap_close(int fd)
{
	size_t i;
	for (i = 0; i < 4; ++i) {
		if (fd != fake_buf_fd[i])
			continue;
		fake_buf_fd[i] = 0;
		break;
	}
	assert(!fail_open || i != 4);
	closed++;
	return 0;
}

int __wrap_logger_open_buffer_from_config(int buf_id, const struct log_config *conf, int open_flags, int *fd)
{
	if (fail_open && buf_id == 2)
		return -EIO;

	assert(conf == &CONF);

	assert(buf_id >= 0);
	assert(buf_id < LOG_ID_MAX);
	assert(!fake_buf_fd[buf_id]);

	if (buf_id == 3) {
		fake_buf_fd[buf_id] = -1;
		return 0;
	}

	const int fake_fd = buf_id + 10;
	*fd = fake_fd;
	fake_buf_fd[buf_id] = fake_fd;
	return 1;
}

// a sample of monologues to make sure longer messages are handled correctly. Stay a while and listen!
static const char *LONG_TEXTS[] = {
	"Kostka Horadrimow, ktora posiadasz, to wielki skarb. Wedlug podan, za jej pomoca mozna odtworzyc Laske Horadrimow! Aby to zrobic, musisz uzyc Kostki tak, jak zwoju. Kiedy sie otworzy, umiesc w niej obie czesci Laski i uzyj mocy transmutacyjnej, jaka w niej zawarto. Na pewno ucieszysz sie takze, gdy ci powiem, ze Kostke mozna wykorzystywac rowniez w inny sposob. Umiesc w niej 5 klejnotow tego samego rodzaju i jakosci, a bedziesz mogl przemienic je w jeden klejnot o wiekszej wartosci. Szesc klejnotow oraz miecz zmienia sie w dlugi miecz z miejscem na osadzenie klejnotu. Mozna takze transmutowac dwa kolczany beltow do kusz w jeden kolczan ze strzalami - albo dwa kolczany ze strzalami w jeden kolczan beltow. Tobie pozostawiam odkrycie innych formul przemiany.",
	"Dziekuje ci wielce za przyjscie mi z pomoca! W dowod wdziecznosci bede dla ciebie za darmo identyfikowal przedmioty. Niestety, nie moglem nic zrobic, aby zapobiec zniszczeniu Tristram. Wyglada na to, ze nasze najgorsze obawy sie sprawdzily. Diablo znowu jest na wolnosci! Jak ci wiadomo, jakis czas temu Pan Grozy zostal zabity w kryptach pod Tristram przez wielkiego wojownika. Kiedy nasz bohater wyszedl z tuneli, opromieniony chwala, przez kilka dni swietowalismy jego zwyciestwo. Ale w ciagu nastepnych tygodni zaczal unikac ludzi i wygladalo na to, ze popadl w depresje. Myslalem, ze moze po prostu nie moze sie otrzasnac po tym, co przeszedl. Z kazdym dniem nasz wybawiciel popadal w wieksza udreke. Pamietam, jak nieraz budzil sie w nocy z krzykiem, wspominajac cos o krainach Wschodu. Pewnego dnia po prostu odszedl, a niedlugo potem Tristram zaatakowaly legiony demonow. Wielu utracilo wtedy zycie, a te diabelskie bestie zostawily mnie, zebym skonal w tej piekielnej klatce. Pewnie to wlasnie pogromca Diablo przechodzil tedy, zanim klasztor opanowalo zlo. Obawiam sie czegos gorszego. Boje sie, ze Diablo zawladnal cialem tego, ktory go zgladzil. Jesli to prawda, to Pan Grozy stanie sie potezniejszy niz kiedykolwiek. Musisz go powstrzymac albo bedziemy zgubieni.",
	"Jestem Akara, najwyzsza kaplanka Siostr Niewidzacego Oka. Witaj w naszym obozie, ale obawiam sie, ze nie mamy ci do zaoferowania nic poza sypiacymi sie murami. Widzisz, na nasz klasztor spadlo okrutne przeklenstwo. Potezna cytadela, w ktorej od pokolen strzeglysmy drogi na wschod zostala opanowana przez demonice o imieniu Andariel. Ciagle trudno mi uwierzyc, ale udalo sie jej przeciagnac na swoja strone inne siostry i wspolnie z nimi wypedzic nas z domu przodkow. Teraz ostatnie obronczynie zakonu albo juz nie zyja, albo zostaly rozproszone. Blagam cie, pomoz nam! Znajdz jakis sposob, aby zdjac klatwe, a przysiegniemy ci wiernosc po wsze czasy. Na pustkowiu znajduje sie miejsce, w ktorym skoncentrowalo sie wiele zlej mocy. Zwiadowczynie Kashyi doniosly mi, ze w pobliskiej jaskini roi sie od mrocznych bestii i potworow z piekla rodem. Obawiam sie, ze te poczwary szykuja sie do ataku na nasz oboz. Jesli naprawde chcesz nam pomoc, to udaj sie do jaskini i zniszcz te plugawe bestie! Niech Niewidzace Oko ma cie w swojej pieczy.",
	"Dobrze cie widziec, szlachetny Paladynie. Dawno juz nie widzialem na Zachodzie zadnego z was. Zaszczytem bedzie dla mnie pomoc ci! Na pewno doszla twych uszu wiesc o tragedii, jaka spadla na miasteczko Tristram! Niektorzy uwazaja, ze Diablo, Pan Grozy, znowu chodzi po swiecie. Nie wiem, czy w to wierzyc, ale kilka tygodniu temu przeszedl tym szlakiem mroczny wedrowiec. Podazal na wschod w strone przeleczy chronionej przez klasztor Lotrzyc. Moze to nic takiego, ale wkrotce potem wrota klasztoru zostaly zamkniete, a kraj zaczely nawiedzac dziwne stwory. Pozostane tu z moja karawana, az zrobi sie bezpieczniej, a bramy zostana otwarte. Mam nadzieje wyruszyc do Lut Gholein zanim nas wszystkich pochlonie cien, jaki padl na Tristram. Jezeli przezyjesz do tego czasu, zabiore cie ze soba. Powinienes tez porozmawiac z Akara. Wyglada na to, ze to ona rzadzi w tym obozie - moze powie ci cos wiecej.",
};

#ifdef USE_ANDROID_MONOTONIC
#define SEC 13
#define NSEC 123456789
int __real_clock_gettime(clockid_t clk_id, struct timespec *tp);
int __wrap_clock_gettime(clockid_t clk_id, struct timespec *tp)
{
	if (clk_id == CLOCK_MONOTONIC) {
		tp->tv_sec = SEC;
		tp->tv_nsec = NSEC;
		return 0;
	}

	return __real_clock_gettime(clk_id, tp);
}
#endif /* USE_ANDROID_MONOTONIC */

static ssize_t writev_ret;
static int errno_to_set;
static bool writev_called;
static const char *used_tag;
static const char *used_msg;
ssize_t __wrap_writev(int fd, const struct iovec *iov, int iovcnt)
{
	int i = 0;
	while (i < 4 && fake_buf_fd[i] != fd)
		++i;
	assert(i != 4);

#ifdef USE_ANDROID_MONOTONIC
	assert(iovcnt == 4);
#else
	assert(iovcnt == 3);
#endif /* USE_ANDROID_MONOTONIC */
	assert(iov[0].iov_len == 1);
	assert(*((char *)iov[0].iov_base) == i + 3);
	assert(iov[1].iov_len == strlen(used_tag) + 1);
	assert(!strcmp(used_tag, (char *)iov[1].iov_base));
	assert(iov[2].iov_len == strlen(used_msg) + 1);
	assert(!strcmp(used_msg, (char *)iov[2].iov_base));
#ifdef USE_ANDROID_MONOTONIC
	assert(iov[3].iov_len == sizeof(struct android_logger_footer));
	struct android_logger_footer *alf = iov[3].iov_base;
	assert(alf->magic == ANDROID_LOGGER_FOOTER_MAGIC);
	assert(alf->sec_sent_mono == SEC);
	assert(alf->nsec_sent_mono == NSEC);
#endif /* USE_ANDROID_MONOTONIC */

	writev_called = true;
	errno = errno_to_set;
	return writev_ret;
}

int main()
{
	fail_open = true;
	__dlog_init_android(&CONF);
	assert(closed == 2);
	assert(!write_to_log);
	assert(!destroy_backend);
	closed = 0;

	fail_open = false;
	__dlog_init_android(&CONF);
	assert(closed == 0);
	assert(write_to_log);
	assert(destroy_backend);

#define F(B, P, M) assert(write_to_log(B, P, "tag", M) == DLOG_ERROR_INVALID_PARAMETER)
	F(LOG_ID_INVALID, DLOG_ERROR       , "xx");
	F(LOG_ID_MAX    , DLOG_ERROR       , "xx");
	F(LOG_ID_MAIN   , DLOG_UNKNOWN     , "xx");
	F(LOG_ID_MAIN   , (log_priority) -7, "xx");
	F(LOG_ID_MAIN   , (log_priority) 23, "xx");
	F(LOG_ID_MAIN   , DLOG_ERROR       , NULL);
#undef F

	used_tag = "tag";
	for (int i = 0; i < 4; ++i) {
		writev_called = false;
		writev_ret = i + 20;
		used_msg = LONG_TEXTS[i];
		assert(write_to_log(i, i + 3, "tag", LONG_TEXTS[i]) == i + 20);
		assert(writev_called);

		writev_called = false;
		writev_ret = -1;
		errno_to_set = i + 10;
		assert(write_to_log(i, i + 3, "tag", LONG_TEXTS[i]) == -(i + 10));
		assert(writev_called);
	}

	// NULL tag is also valid, in which case it's implicitly ""
	used_tag = "";
	used_msg = LONG_TEXTS[0];
	writev_ret = 1234;
	assert(write_to_log(0, 3, NULL, LONG_TEXTS[0]) == 1234);

	destroy_backend();
	assert(closed + 1 == LOG_ID_MAX);
}
