// C
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

// DLog
#include <dlog.h>
#include <libdlog.h>
#include <logconfig.h>

void __log_limiter_update(struct log_config *config) { }
void __log_limiter_destroy(void) { }
void __dynamic_config_destroy() { }

static bool destroyed;
void destroy()
{
	destroyed = true;
}

int wtl_android(log_id_t buf_id, log_priority pri, const char *tag, const char *msg) { return 45835705; }
int wtl_pipe(log_id_t buf_id, log_priority pri, const char *tag, const char *msg) { return 0xDECC16; }

void __dlog_init_pipe(const struct log_config *conf) { write_to_log = wtl_pipe; }
void __dlog_init_android(const struct log_config *conf)
{
	write_to_log = wtl_android;
	destroy_backend = destroy;
}

static struct log_config CONFIG;
static bool fail_conf_read;
static bool conf_read_called;
int __wrap_log_config_read(struct log_config *config)
{
	conf_read_called = true;
	config->begin = config->last = NULL;

	if (fail_conf_read)
		return -1;

	log_config_copy(config, &CONFIG);
	return 0;
}

int limiter_ret;
int __log_limiter_pass_log(const char* tag, int prio) { return limiter_ret; }

static bool limiter_created;
int __log_limiter_create(struct log_config *config)
{
	limiter_created = true;
	return 1;
}

static bool use_dynamic_conf;
bool __dynamic_config_create(struct log_config *config) { return use_dynamic_conf; }

static bool dynamic_config_called;
void __dynamic_config_update() { dynamic_config_called = true; }

bool fail_snprintf;
int __wrap_snprintf(char *str, size_t size, const char *format, ...)
{
	if (fail_snprintf && !strcmp(format, "enable_%s")) {
		errno = -ENAMETOOLONG;
		return -1;
	}

	va_list va;
	va_start(va, format);
	const int ret = vsnprintf(str, size, format, va);
	va_end(va);
	return ret;
}

int dlog_vprint_wrap_va_list(log_priority pri, const char *tag, const char *msg, ...)
{
	va_list ap;
	va_start(ap, msg);

	const int ret = dlog_vprint(pri, tag, msg, ap);

	va_end(ap);
	return ret;
}

void tct_tests()
{
	/* These are a copy of TCT tests:
	 * repository: test/tct/native/api on tizen.org
	 * commit: aa0758c84abc5a10f60df68f4f86efe7dc1216c6, 2019-01-21
	 *
	 * Most of the TCT code for DLog is boilerplate setting up the
	 * framework and such. Since that is quite worthless, this file
	 * only copies the meritum. In particular, there are multiple
	 * calls to dlog_print for reporting test results in the TCT;
	 * it is important to keep this in mind when comparing how much
	 * dlog_print seems to get called in TCT code vs the handful of
	 * lines present in this file, since actual test cases consist
	 * of just the latter.
	 *
	 * The other checks earlier in this file should already catch
	 * everything the TCT tests do but it's good to work with the
	 * exact params used in the TCT tests to handle any edge cases
	 * that might appear and, more importantly, to have this done
	 * at build time since the actual TCT suite is a total hassle
	 * to work with, especially given its simplicity.
	 *
	 * Note that we still use the emulated functions since the real
	 * TCT test is done on a live system; the build environment has
	 * nowhere to actually send the logs and the tests only really
	 * check for parameter correctness. */

	// src/itc/dlog/ITs-dlog.c : ITc_dlog_dlog_print_p
	assert(dlog_print(DLOG_DEBUG, "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_print(DLOG_INFO , "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_print(DLOG_WARN , "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_print(DLOG_ERROR, "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);

	// src/itc/dlog/ITs-dlog.c : ITc_dlog_dlog_vprint_p
	assert(dlog_vprint_wrap_va_list(DLOG_DEBUG, "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_vprint_wrap_va_list(DLOG_INFO , "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_vprint_wrap_va_list(DLOG_WARN , "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);
	assert(dlog_vprint_wrap_va_list(DLOG_ERROR, "DLOG_TEST", "[Dlog_ITC] Dlog Test Message\n") >= 0);

	// src/utc/dlog/utc-SystemFW-dlog-print-func.c
	assert(dlog_print(DLOG_INFO, "DLOG_TEST", "dlog test message for tct-mgr\n") > 0);
	assert(dlog_print(-1, NULL, NULL) < 1);

	// src/utc/dlog/utc-SystemFW-dlog-vprint-func.c
	assert(dlog_vprint_wrap_va_list(DLOG_INFO, "DLOG_TEST", "dlog test message for tct-mgr\n") > 0);
	assert(dlog_vprint_wrap_va_list(-1, NULL, NULL) < 1);
}

int main()
{
	fail_conf_read = true;
	__configure();
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NOT_PERMITTED);
	assert(write_to_log != wtl_android);
	assert(write_to_log != wtl_pipe);
	fail_conf_read = false;
	__dlog_fini();

	__configure();
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NOT_PERMITTED);
	assert(write_to_log != wtl_android);
	assert(write_to_log != wtl_pipe);
	conf_read_called = false;
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NOT_PERMITTED);
	assert(conf_read_called); // should retry reinitializing
	__dlog_fini();

	log_config_set(&CONFIG, "backend", "Hej, zamawiam Bentoye. Czekam do 11:15.");
	__configure();
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NOT_PERMITTED);
	assert(write_to_log != wtl_pipe);
	assert(write_to_log != wtl_android);
	__dlog_fini();


	log_config_set(&CONFIG, "backend", "pipe");
	log_config_set(&CONFIG, "limiter", "0");
	__configure();
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == 0xDECC16);
	assert(!dynamic_config_called);
	assert(write_to_log == wtl_pipe);
	assert(!limiter_created);
	__dlog_fini();

	log_config_set(&CONFIG, "backend", "logger");
	log_config_set(&CONFIG, "limiter", "1");
	log_config_set(&CONFIG, "debugmode", "0");
	use_dynamic_conf = true;
	limiter_ret = 1;
	__configure();
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == 45835705);
	assert(__dlog_print(LOG_ID_MAIN, DLOG_ERROR, "tag", "msg") == 45835705);
	assert(write_to_log != wtl_pipe); // ceci n'est pas une pipe
	assert(write_to_log == wtl_android);
	assert(limiter_created);
	assert(dynamic_config_called);

	limiter_ret = 0;
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == 45835705);
	assert(__dlog_print(LOG_ID_MAIN, DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NONE);
	limiter_ret = -1;
	assert(dlog_print(DLOG_ERROR, "tag", "msg") == 45835705);
	assert(__dlog_print(LOG_ID_MAIN, DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NONE);

	assert(__dlog_print((log_id_t) -1, DLOG_ERROR, "tag", "msg") == DLOG_ERROR_INVALID_PARAMETER);
	assert(__dlog_print(LOG_ID_MAIN, DLOG_ERROR, NULL , "msg") == DLOG_ERROR_INVALID_PARAMETER);
	assert(__dlog_print(LOG_ID_MAIN, DLOG_DEBUG, "tag", "msg") == DLOG_ERROR_NONE); // limited

	limiter_ret = 1;
	__dlog_fini();

	log_config_set(&CONFIG, "enable_system", "0");
	__configure();
	assert(__dlog_print(LOG_ID_SYSTEM, DLOG_ERROR, "tag", "msg") == DLOG_ERROR_NONE);
	__dlog_fini();

	fail_snprintf = true;
	__configure();
	assert(__dlog_print(LOG_ID_SYSTEM, DLOG_ERROR, "tag", "msg") == 45835705);
	fail_snprintf = false;
	__dlog_fini();

	__configure();
	tct_tests();

	destroyed = false;
	__dlog_fini();
	assert(destroyed);
}
