#include <stdio.h>
#include <assert.h>
#include <time.h>

#include <unistd.h>

#include <logcommon.h>
#include <queued_entry.h>

int time_cmp(struct timespec x, int sec, int nsec)
{
	if (x.tv_sec  > sec)  return +1;
	if (x.tv_sec  < sec)  return -1;
	if (x.tv_nsec > nsec) return +1;
	if (x.tv_nsec < nsec) return -1;
	return 0;
}

int main()
{
	int prio = 5;
	char tag[] = "TEST_TAG";
	char msg[] = "Test message with a newline \n and some special characters \e[31m in colour";
	struct pipe_logger_entry *ple = alloca(LOG_MAX_PAYLOAD_SIZE + sizeof *ple);
	struct logger_entry *le = alloca(LOG_MAX_PAYLOAD_SIZE + sizeof *le);
	struct timespec ts_pre, ts_post;

	clock_gettime(CLOCK_MONOTONIC, &ts_pre);
	create_pipe_message(ple, prio, tag, msg);
	parse_pipe_message(ple, le, ple->len);
	clock_gettime(CLOCK_MONOTONIC, &ts_post);

	assert(le->pid == getpid());
	assert(le->tid == gettid());
	assert(le->priority == prio);
	assert(!strcmp(le->msg, tag));
	assert(!strcmp(le->msg + sizeof(tag), msg));
	assert(time_cmp(ts_pre,  le->sec_sent_mono, le->nsec_sent_mono) <= 0);
	assert(time_cmp(ts_post, le->sec_sent_mono, le->nsec_sent_mono) >= 0);

	static const int SAFE_SIZE = ((LOG_MAX_PAYLOAD_SIZE - 1 /* prio */) / 2 /* split over tag and msg */) - 1 /* null delimiter */;
	char random_tag[SAFE_SIZE];
	char random_msg[SAFE_SIZE];
	random_tag[SAFE_SIZE - 1] = random_msg[SAFE_SIZE - 1] = '\0';

	int i = 100;
	while (i--) {
		int j = SAFE_SIZE - 1;
		while (j--) {
			random_msg[j] = rand() & 255;
			random_tag[j] = rand() & 255;
		}
		prio = (rand() % (DLOG_PRIO_MAX - DLOG_VERBOSE)) + DLOG_VERBOSE;

		clock_gettime(CLOCK_MONOTONIC, &ts_pre);
		create_pipe_message(ple, prio, random_tag, random_msg);
		parse_pipe_message(ple, le, ple->len);
		clock_gettime(CLOCK_MONOTONIC, &ts_post);
		assert(le->priority == prio);
		assert(!strcmp(le->msg, random_tag));
		assert(!strcmp(le->msg + strlen(random_tag) + 1, random_msg));
		assert(le->pid == getpid());
		assert(le->tid == gettid());
		assert(time_cmp(ts_pre,  le->sec_sent_mono, le->nsec_sent_mono) <= 0);
		assert(time_cmp(ts_post, le->sec_sent_mono, le->nsec_sent_mono) >= 0);
	}

	return 0;
}
