#include <logprint.h>
#include <queued_entry.h>
#include <stdio.h>
#include <assert.h>

int main()
{
	char buffer[1024];
	int len;
	struct logger_entry * sle = alloca(1024);

	// No priority delimiter '>'.
	len = snprintf(buffer, sizeof buffer, "<10 Jan 02 03:04:05 progname: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// No priority.
	len = snprintf(buffer, sizeof buffer, "<>Jan 02 03:04:05 progname: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Priority is garbage.
	len = snprintf(buffer, sizeof buffer, "<XX>Jan 02 03:04:05 progname: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Priority is OVER 9000
	len = snprintf(buffer, sizeof buffer, "<999999999999999999999999999999999>Jan 02 03:04:05 progname: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// No timestamp
	len = snprintf(buffer, sizeof buffer, "<10>");
	assert(parse_syslog_datagram(buffer, len, sle));

	// No progname
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 ");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Progname cut
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 prog");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Progname not alphanumeric
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 @_@: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Pid not a number
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname[pid]: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Pid bracket not closed
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname[678: msg");
	assert(parse_syslog_datagram(buffer, len, sle));

	// No message
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname:");
	assert(parse_syslog_datagram(buffer, len, sle));

	// No message, version with space
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname: ");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Cut date
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:XX");
	assert(parse_syslog_datagram(buffer, len, sle));

	// Valid
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname[678]: msg");
	assert(!parse_syslog_datagram(buffer, len, sle));

	assert(sle->pid == 678);
	assert(sle->priority == DLOG_FATAL);
	assert(!strcmp("SYSLOG_USER", log_buffer_get_tag(sle)));
	assert(!strcmp("msg", log_buffer_get_message(sle)));

	// Valid
	len = snprintf(buffer, sizeof buffer, "<10>Jan 02 03:04:05 progname: msg");
	assert(!parse_syslog_datagram(buffer, len, sle));

	assert(sle->pid == 0);
	assert(sle->priority == DLOG_FATAL);
	assert(!strcmp("SYSLOG_USER", log_buffer_get_tag(sle)));
	assert(!strcmp("msg", log_buffer_get_message(sle)));

	return 0;
}
