#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <ptrs_list.h>

#define NELEMS(arr) (sizeof(arr)/sizeof(arr[0]))

static void *pointers_to_free[10];

void __real_free(void *ptr);
void __wrap_free(void *ptr)
{
	for (size_t i = 0; i < NELEMS(pointers_to_free); ++i)
		if (ptr == pointers_to_free[i])
			pointers_to_free[i] = NULL;

	__real_free(ptr);
}

static int fail_calloc = 0;
void *__real_calloc(size_t nmemb, size_t size);
void *__wrap_calloc(size_t nmemb, size_t size)
{
	bool fail_now = fail_calloc % 2;
	fail_calloc >>= 1;
	return fail_now ? NULL : __real_calloc(nmemb, size);
}

static elem_value fail_copy_on = NULL;
static elem_value copy_cb(elem_value value, void *_)
{
	return fail_copy_on == value ? NULL : value;
}

static bool letters_cleared[3];
static void clear_cb(elem_value value, void *_)
{
	letters_cleared[((char *)value)[0] - 'a'] = true;
}

int main(int argc, char ** argv)
{
	list_head head = NULL;
	assert(list_count(head) == 0);

	void *const elems[] = {"a", "b", "c"};
	for (size_t i = 0; i < NELEMS(elems); i++) {
		assert(list_add(&head, elems[i]));
		assert(list_count(head) == i + 1);
	}

	fail_calloc = 7;
	for (size_t i = 0; i < NELEMS(elems); i++) {
		assert(!list_add(&head, elems[i]));
		assert(list_count(head) == NELEMS(elems));
	}

	list_head iter = head;
	for (size_t i = 0; i < NELEMS(elems); ++i, list_next(&iter))
		assert(list_at(iter) == elems[NELEMS(elems) - 1 - i]);
	assert(iter == NULL);

	list_head head2 = list_map(head, NULL, copy_cb, clear_cb);
	assert(list_count(head2) == 3);

	fail_copy_on = elems[0];
	letters_cleared[0] = letters_cleared[1] = letters_cleared[2] = false;
	list_map(head, NULL, copy_cb, clear_cb);
	assert(!letters_cleared[0] && letters_cleared[1] && letters_cleared[2]);
	fail_copy_on = NULL;

	fail_calloc = 4;
	letters_cleared[0] = letters_cleared[1] = letters_cleared[2] = false;
	list_map(head, NULL, copy_cb, clear_cb);
	assert(letters_cleared[0] && letters_cleared[1] && letters_cleared[2]);

	list_remove(&head, elems[0]); //remove tail
	assert(list_count(head) == 2);
	assert(list_at(head) == elems[2]);
	list_remove(&head, elems[2]); //remove head
	assert(list_count(head) == 1);
	assert(list_at(head) == elems[1]);
	list_remove(&head, elems[1]); //remove last
	assert(list_count(head) == 0);
	assert(head == NULL);

	list_clear(&head2);
	assert(list_count(head2) == 0);
	assert(head2 == NULL);

	iter = NULL;
	list_next(&iter);
	assert(iter == NULL);

	for (size_t i = 0; i < NELEMS(pointers_to_free); ++i) {
		pointers_to_free[i] = malloc(i + 5);
		list_add(&head, pointers_to_free[i]);
	}
	list_clear_free_contents(&head);
	assert(head == NULL);
	for (size_t i = 0; i < NELEMS(pointers_to_free); ++i)
		assert(pointers_to_free[i] == NULL);

	return 0;
}
